#!/usr/bin/env python
"""
:Author Patrik Valkovic
:Created 16.08.2017 19:33
:Licence GNUv3
Part of grammpy

"""

from .IsMethodsRuleExtension import IsMethodsRuleExtension


class ActivtableRule(IsMethodsRuleExtension):
    _active = True
